@@ -4,8 +4,8 @@
 
 
 # Configuration directories and files
-SourceDirectory: /var/lib/jenkins/workspace/build-galera-source/galera-source
-BuildDirectory: /var/lib/jenkins/workspace/build-galera-source/galera-source
+SourceDirectory: /var/lib/jenkins/workspace/aws-galera-4-sourcetar
+BuildDirectory: /var/lib/jenkins/workspace/aws-galera-4-sourcetar
 
 # Where to place the cost data store
 CostDataFile: 
@@ -16,6 +16,9 @@ Site: master
 # Build name is osname-revision-compiler, i.e. Linux-2.4.2-2smp-c++
 BuildName: Linux-c++
 
+# Subprojects
+LabelsForSubprojects: 
+
 # Submission information
 IsCDash: 
 CDashVersion: 
@@ -33,7 +36,7 @@ ScpCommand: /usr/bin/scp
 NightlyStartTime: 00:00:00 EDT
 
 # Commands for the build/test/submit cycle
-ConfigureCommand: "/usr/bin/cmake" "/var/lib/jenkins/workspace/build-galera-source/galera-source"
+ConfigureCommand: "/usr/bin/cmake" "/var/lib/jenkins/workspace/aws-galera-4-sourcetar"
 MakeCommand: /usr/bin/cmake --build . --config "${CTEST_CONFIGURATION_TYPE}" -- -i
 DefaultCTestConfigurationType: Release
 
@@ -52,6 +55,7 @@ SVNUpdateOptions:
 
 # Git options
 GITCommand: /usr/bin/git
+GITInitSubmodules: 
 GITUpdateOptions: 
 GITUpdateCustom: 
 
@@ -69,7 +73,7 @@ UpdateType: git
 
 # Compiler info
 Compiler: /usr/bin/c++
-CompilerVersion: 5.4.0
+CompilerVersion: 7.5.0
 
 # Dynamic analysis (MemCheck)
 PurifyCommand: 
