@@ -411,6 +411,7 @@ namespace galera
         {
             if (gu_unlikely(purge_seqno != -1))
             {
+                assert(purge_seqno <= last_committed());
                 service_thd_.report_last_committed(purge_seqno);
             }
         }
@@ -465,8 +466,6 @@ namespace galera
                                          const TrxHandleSlavePtr&);
         wsrep_status_t cert_and_catch   (TrxHandleMaster*,
                                          const TrxHandleSlavePtr&);
-        wsrep_status_t cert_for_aborted (const TrxHandleSlavePtr&);
-
         // Enter apply monitor for local transaction. Return true
         // if apply monitor was grabbed.
         bool enter_apply_monitor_for_local(TrxHandleMaster&,
