@@ -525,7 +525,8 @@ void gu::ssl_register_params(gu::Config& conf)
              gu::Config::Flag::type_bool);
     conf.add(gu::conf::ssl_compression,
              gu::Config::Flag::read_only |
-             gu::Config::Flag::type_bool);
+             gu::Config::Flag::type_bool |
+             gu::Config::Flag::deprecated);
     conf.add(gu::conf::ssl_key,
              gu::Config::Flag::read_only);
     conf.add(gu::conf::ssl_cert,
@@ -595,6 +596,12 @@ void gu::ssl_init_options(gu::Config& conf)
             log_info << "disabling SSL compression";
             sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
         }
+        else
+        {
+            log_warn << "SSL compression is not effective. The option "
+                     << conf::ssl_compression << " is deprecated and "
+                     << "will be removed in future releases.";
+        }
         conf.set(conf::ssl_compression, compression);
 
 
