@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2014 Codership Oy <info@codership.com> */
+/* Copyright (C) 2011-2016 Codership Oy <info@codership.com> */
 
 #ifndef _GARB_RECV_LOOP_HPP_
 #define _GARB_RECV_LOOP_HPP_
@@ -8,6 +8,7 @@
 
 #include <gu_throw.hpp>
 #include <gu_asio.hpp>
+#include <common.h> // COMMON_BASE_DIR_KEY
 
 #include <pthread.h>
 
@@ -48,11 +49,17 @@ private:
         ParseOptions(gu::Config& cnf, const std::string& opt)
         {
             cnf.parse(opt);
+            gu::ssl_init_options(cnf);
         }
     }
         parse_;
 
-    Gcs           gcs_;
+    Gcs gcs_;
+
+    gu::UUID    uuid_;
+    gu::seqno_t seqno_;
+    int         proto_;
+
 }; /* RecvLoop */
 
 } /* namespace garb */
