@@ -68,6 +68,11 @@ namespace gcomm
          */
         static std::string const SocketRecvBufSize;
 
+        /*!
+         * @brief Socket send buffer size in bytes.
+         */
+        static std::string const SocketSendBufSize;
+
         /*!
          * @brief GMCast scheme for transport URI ("gmcast")
          */
@@ -120,7 +125,7 @@ namespace gcomm
          *
          * This parameter controls multicast packet TTL. By default it
          * is set to 1 and usually it should not be changed unless
-         * adviced so. This means that multicast is limited to single LAN
+         * advised so. This means that multicast is limited to single LAN
          * segment.
          */
         static std::string const GMCastMCastTTL;
@@ -147,7 +152,7 @@ namespace gcomm
         /*!
          * @brief Isolate node from peers
          *
-         * Setting this value to 'true' closes all conections
+         * Setting this value to 'true' closes all connections
          * and will prevent forming of new connections until
          * value is set again to 'false'. This parameter should be
          * used for testing purposes only and it will not be visible
@@ -292,7 +297,7 @@ namespace gcomm
          * This parameter controls whether EVS is allowed to aggregate
          * several user messages into one message. By default this option
          * is enabled and there should be no need to disable it unless
-         * adviced so.
+         * advised so.
          */
         static std::string const EvsUseAggregate;
 
@@ -426,6 +431,7 @@ namespace gcomm
         };
 
         static size_t check_recv_buf_size(const std::string& val);
+        static size_t check_send_buf_size(const std::string& val);
     };
 
 
