@@ -37,9 +37,9 @@ static bool check_tcp_uri(const gu::URI& uri)
             uri.get_scheme() == gu::scheme::ssl);
 }
 
-static std::string get_scheme(bool use_ssl)
+static std::string get_scheme(bool use_ssl, bool dynamic_socket)
 {
-    if (use_ssl == true)
+    if (use_ssl == true && not dynamic_socket)
     {
         return gu::scheme::ssl;
     }
@@ -81,13 +81,21 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
                           param<int>(conf_, uri, Conf::GMCastSegment, "0"),
                           0, 255)),
     my_uuid_      (my_uuid ? *my_uuid : UUID(0, 0)),
+    dynamic_socket_ (conf_.has(gu::conf::socket_dynamic) ?
+                     param<bool>(conf_, uri, gu::conf::socket_dynamic,
+                                 "false") :
+                     false),
+#ifdef GALERA_HAVE_SSL
     use_ssl_      (param<bool>(conf_, uri, gu::conf::use_ssl, "false")),
+#else
+    use_ssl_(),
+#endif // GALERA_HAVE_SSL
     // @todo: technically group name should be in path component
     group_name_   (param<std::string>(conf_, uri, Conf::GMCastGroup, "")),
     listen_addr_  (
         param<std::string>(
             conf_, uri, Conf::GMCastListenAddr,
-            get_scheme(use_ssl_) + "://0.0.0.0")), // how to make it IPv6 safe?
+            get_scheme(use_ssl_, dynamic_socket_) + "://0.0.0.0")), // how to make it IPv6 safe?
     initial_addrs_(),
     mcast_addr_   (param<std::string>(conf_, uri, Conf::GMCastMCastAddr, "")),
     bind_ip_      (""),
@@ -95,7 +103,7 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
                        Conf::GMCastMCastTTL,
                        param<int>(conf_, uri, Conf::GMCastMCastTTL, "1"),
                        1, 256)),
-    listener_     (0),
+    listener_     (),
     mcast_        (),
     pending_addrs_(),
     remote_addrs_ (),
@@ -143,7 +151,7 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
     catch (gu::Exception&)
     {
         /* most probably no scheme, try to append one and see if it succeeds */
-        listen_addr_ = uri_string(get_scheme(use_ssl_), listen_addr_);
+        listen_addr_ = uri_string(get_scheme(use_ssl_, dynamic_socket_), listen_addr_);
         gu_trace(gu::URI uri(listen_addr_));
     }
 
@@ -188,11 +196,10 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
 
     listen_addr_ = gu::net::resolve(listen_addr_).to_string();
     // resolving sets scheme to tcp, have to rewrite for ssl
-    if (use_ssl_ == true)
+    if (use_ssl_ == true && not dynamic_socket_)
     {
         listen_addr_.replace(0, 3, gu::scheme::ssl);
     }
-
     std::set<std::string>::iterator iaself(initial_addrs_.find(listen_addr_));
     if (iaself != initial_addrs_.end())
     {
@@ -274,7 +281,7 @@ void gcomm::GMCast::set_initial_addr(const gu::URI& uri)
             }
         }
 
-        std::string initial_uri = uri_string(get_scheme(use_ssl_), host, port);
+        std::string initial_uri = uri_string(get_scheme(use_ssl_, dynamic_socket_), host, port);
         std::string initial_addr;
         try
         {
@@ -285,9 +292,8 @@ void gcomm::GMCast::set_initial_addr(const gu::URI& uri)
             log_warn << "Failed to resolve " << initial_uri;
             continue;
         }
-
         // resolving sets scheme to tcp, have to rewrite for ssl
-        if (use_ssl_ == true)
+        if (use_ssl_ == true && not dynamic_socket_)
         {
             initial_addr.replace(0, 3, gu::scheme::ssl);
         }
@@ -373,8 +379,7 @@ void gcomm::GMCast::close(bool force)
 
     gcomm_assert(listener_ != 0);
     listener_->close();
-    delete listener_;
-    listener_ = 0;
+    listener_.reset();
 
     segment_map_.clear();
     for (ProtoMap::iterator
@@ -1799,7 +1804,7 @@ void gcomm::GMCast::add_or_del_addr(const std::string& val)
     if (val.compare(0, 4, "add:") == 0)
     {
         gu::URI uri(val.substr(4));
-        std::string addr(gu::net::resolve(uri_string(get_scheme(use_ssl_),
+        std::string addr(gu::net::resolve(uri_string(get_scheme(use_ssl_, dynamic_socket_),
                                                      uri.get_host(),
                                                      uri.get_port())).to_string());
         log_info << "inserting address '" << addr << "'";
