@@ -22,7 +22,9 @@ uint8_t     const GCS_VOTE_POLICY_DEFAULT(0);
 
 void gcs_group_register(gu::Config* cnf)
 {
-    cnf->add(GCS_VOTE_POLICY_KEY);
+    cnf->add(GCS_VOTE_POLICY_KEY,
+             gu::Config::Flag::read_only |
+             gu::Config::Flag::type_integer);
 }
 
 const char* gcs_group_state_str[GCS_GROUP_STATE_MAX] =
@@ -151,7 +153,10 @@ group_nodes_init (const gcs_group_t* group, const gcs_comp_msg_t* comp)
 }
 
 /* Free nodes array */
-static void
+#ifndef GCS_CORE_TESTING
+static
+#endif // GCS_CORE_TESTING
+void
 group_nodes_free (gcs_group_t* group)
 {
     int i;
@@ -281,7 +286,8 @@ group_redo_last_applied (gcs_group_t* group)
         group->last_node    = last_node;
     }
 
-    log_debug << "final last_applied: " << group->last_applied;
+    log_debug << "final last_applied on " << group->nodes[group->my_idx].name
+              << "): " << group->last_applied;
 }
 
 static void
@@ -694,14 +700,6 @@ gcs_group_handle_uuid_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
     return group->state;
 }
 
-static void group_print_state_debug(gcs_state_msg_t* state)
-{
-    size_t str_len = 1024;
-    char state_str[str_len];
-    gcs_state_msg_snprintf (state_str, str_len, state);
-    gu_info ("%s", state_str);
-}
-
 gcs_group_state_t
 gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
 {
@@ -722,8 +720,6 @@ gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
                          msg->sender_idx, gcs_state_msg_name(state));
                 gu_debug("%s", state_str);
 
-                if (gu_log_debug) group_print_state_debug(state);
-
                 gcs_node_record_state (&group->nodes[msg->sender_idx], state);
                 group_post_state_exchange (group);
             }
@@ -736,8 +732,6 @@ gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
                           GU_UUID_ARGS(&group->state_uuid));
                 gu_debug ("%s", state_str);
 
-                if (gu_log_debug) group_print_state_debug(state);
-
                 gcs_state_msg_destroy (state);
             }
         }
@@ -827,8 +821,8 @@ gcs_group_handle_last_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
         group_redo_last_applied (group);
 
         if (old_val < group->last_applied) {
-            gu_debug ("New COMMIT CUT %lld after %lld from %d",
-                      (long long)group->last_applied,
+            gu_debug ("New COMMIT CUT %lld on %d after %lld from %d",
+                      (long long)group->last_applied, group->my_idx,
                       (long long)gtid.seqno(), msg->sender_idx);
             return group->last_applied;
         }
@@ -1749,7 +1743,10 @@ group_select_donor (gcs_group_t* group,
 void
 gcs_group_ignore_action (gcs_group_t* group, struct gcs_act_rcvd* act)
 {
-//    if (act->act.type <= GCS_ACT_STATE_REQ) {
+    gu_debug("Ignoring action: buf: %p, len: %zd, type: %d, sender: %d, "
+             "seqno: %lld", act->act.buf, act->act.buf_len, act->act.type,
+             act->sender_idx, act->id);
+
     if (act->act.type <= GCS_ACT_CCHANGE) {
         gcs_gcache_free (group->cache, act->act.buf);
     }
@@ -1786,31 +1783,58 @@ gcs_group_handle_state_request (gcs_group_t*         group,
     gu::GTID ist_gtid;
     int str_version = 1; // actually it's 0 or 1.
 
-    if (act->act.buf_len > (ssize_t)donor_name_len &&
-        donor_name[donor_name_len + 0] == 'V') {
+    if (act->act.buf_len > (ssize_t)(donor_name_len + 1) &&
+        donor_name[donor_name_len] == 'V') {
         str_version = (int)donor_name[donor_name_len + 1];
     }
 
     if (str_version >= 2) {
-        size_t offset(donor_name_len + 2);
+        ssize_t const ist_offset(donor_name_len + 2);
+        ssize_t const sst_offset(ist_offset + gu::GTID::serial_size());
 
         try
         {
-            offset = ist_gtid.unserialize(act->act.buf, act->act.buf_len,offset);
+            if (act->act.buf_len < sst_offset)
+            {
+                gu_throw_error(EINVAL) << "Request message too short: "
+                                       << act->act.buf_len << " < "
+                                       << sst_offset;
+            }
+
+            ssize_t const offset
+                (ist_gtid.unserialize(act->act.buf, act->act.buf_len,
+                                      ist_offset));
+            if (offset != sst_offset)
+            {
+                gu_throw_error(EINVAL) << "Actual SST offset " << offset
+                                       << " does not match expected "
+                                       << sst_offset;
+            }
         }
-        catch (gu::Exception& e) {
-            log_warn << "Malformed state transfer request: " << e.what()
-                     << " Ignoring";
-            gcs_group_ignore_action(group, act);
-            return 0;
+        catch (gu::Exception& e)
+        {
+            if (group->my_idx == joiner_idx)
+            {
+                log_fatal << "Failed to form State Transfer Request: "
+                          << e.what();
+                act->id = -ENOTRECOVERABLE;
+                return act->act.buf_len;
+            }
+            else
+            {
+               log_warn << "Malformed State Transfer Request: " << e.what()
+                        << " Ignoring";
+                gcs_group_ignore_action(group, act);
+                return 0;
+            }
         }
 
         // change act.buf's content to original version.
         // and it's safe to change act.buf_len
         ::memmove((char*)act->act.buf + donor_name_len,
-                  (char*)act->act.buf + offset,
-                  act->act.buf_len - offset);
-        act->act.buf_len -= offset - donor_name_len;
+                  (char*)act->act.buf + sst_offset,
+                  act->act.buf_len - sst_offset);
+        act->act.buf_len -= sst_offset - donor_name_len;
     }
 
     assert (GCS_ACT_STATE_REQ == act->act.type);
